import { Component, ViewChild, DoCheck } from '@angular/core';

import { acSection, Result } from 'jalhyd';

import { SectionCanvasComponent } from '../section-canvas/section-canvas.component';
import { SectionResults } from '../../results/section-results';
import { ApplicationSetupService } from '../../services/app-setup/app-setup.service';
import { CalculatorResults } from '../../results/calculator-results';

@Component({
    selector: 'section-results',
    templateUrl: './section-results.component.html',
    styles: [`
    .result_label {
        text-align: right;
        padding-top:10px;
        padding-bottom:10px;
        padding-right:10px;   
    }
    .result_value {
        text-align: center;
        padding-left:30px;   
        padding-right:30px;   
    }
    .result_id_0 {
        background-color: #f0f0f0;
    }
    .result_id_2 {
        font-weight: bold;
    }
    `
    ]
})
export class SectionResultsComponent implements DoCheck {
    /**
    * résultats non mis en forme
    */
    private _results: SectionResults;

    constructor(private appSetupService: ApplicationSetupService) { }

    private static labelColors: { [key: string]: any; } = {
        "Hs": { r: 255, g: 0, b: 0 },
        "Hsc": { r: 0, g: 0, b: 255 },
        "Yn": { r: 16, g: 128, b: 16 },
        "Yf": { r: 128, g: 128, b: 128 },
        "Yt": { r: 0, g: 0, b: 0 },
        "Yc": { r: 255, g: 128, b: 0 },
        "Yco": { r: 255, g: 0, b: 255 },
    };

    private _doUpdate: boolean = false;

    @ViewChild(SectionCanvasComponent)
    private _sectionCanvas: SectionCanvasComponent;

    public set results(rs: CalculatorResults[]) {
        this._results = undefined;
        if (rs != undefined)
            for (const r of rs) {
                if (r instanceof SectionResults) {
                    this._results = r;
                    break;
                }
            }
        this.updateView();
    }

    public updateView() {
        if (this._sectionCanvas != undefined)
            this._sectionCanvas.reset();

        if (this._results != undefined)
            this._doUpdate = this._results.hasResults;
    }

    /** 
     * appelé pour gérer les changements non détectés par Angular
     */
    public ngDoCheck() {
        if (this._doUpdate)
            this._doUpdate = !this.updateResults();
    }

    private updateResults() {
        if (this._results != undefined && this._sectionCanvas != undefined) {
            const nDigits = this.appSetupService.displayDigits;

            for (let r of this._results.results) {
                const v: Result = r["value"];
                const l = r["label"];

                const drawLabel = r["drawLabel"];
                if (drawLabel != undefined && v.vCalc != undefined)
                    this._sectionCanvas.addLevel(v.vCalc, drawLabel + " = " + v.vCalc.toFixed(nDigits), SectionResultsComponent.labelColors[drawLabel]);
            }
            this._sectionCanvas.section = this._results.section;
            return true;
        }
        return false;
    }

    private get hasResults(): boolean {
        return this._results != undefined && this._results.hasResults;
    }

    private get sectionResults() {
        return this._results.results;
    }

    private getResultClass(i: number) {
        return "result_id_" + String(i & 1);
    }
}
