import { Component, Input } from "@angular/core";

import { ResultElement } from 'jalhyd';

import { ApplicationSetupService } from "../../services/app-setup/app-setup.service";
import { InternationalisationService } from "../../services/internationalisation/internationalisation.service";
import { OnChanges } from "@angular/core/src/metadata/lifecycle_hooks";

@Component({
    selector: "single-result",
    templateUrl: "./single-result.component.html"
})
export class SingleResultComponent implements OnChanges {
    @Input("result")
    private _result: ResultElement;

    /**
     * valeur à afficher
     */
    private _value: string;

    /**
     * true si la valeur est en erreur
     */
    private _hasError: boolean;

    /**
     * code HTML du tooltip
     */
    private _htmlTooltip: string;

    /**
     * true si pas de texte à afficher dans le tooltip
     */
    private _emptyTooltip: boolean = false;

    constructor(
        private appSetupService: ApplicationSetupService,
        private intlService: InternationalisationService
    ) { }

    /**
     * appelé quand les @Input changent
     */
    ngOnChanges() {
        this.updateTooltip();
    }

    private updateTooltip() {
        // valeur à afficher

        const nDigits = this.appSetupService.displayDigits;
        const r: ResultElement = this._result;
        this._hasError = r == undefined || r.vCalc == undefined;
        this._value = this._hasError ? " " : this._value = r.vCalc.toFixed(nDigits);

        // texte du tooltip

        let res = "";

        if (this._result != undefined)
            for (let m of this._result.log.messages) {
                if (res.length > 0)
                    res += "<br/>";
                res += this.intlService.localizeMessage(m);
            }

        this._htmlTooltip = res;
        this._emptyTooltip = this._htmlTooltip.length == 0;
    }

    private get value(): string {
        return this._value;
    }

    private get hasError() {
        return this._hasError && !this._emptyTooltip;
    }

    private get htmlTooltip(): string {
        return this._htmlTooltip;
    }

    private get tooltipDisabled(): boolean {
        return this._emptyTooltip;
    }
}
